@@ -1,8 +1,8 @@
 import numpy as np
 import datetime as dt
 import collections as collec
-import datastructures as ds
-from glob import outputwidth, version
+import clust.scripts.datastructures as ds
+from clust.scripts.glob import outputwidth, version
 import pandas as pd
 
 
@@ -80,7 +80,7 @@ def generateinitialmessage():
     res = '\n' + topline()
     tmptxt = 'Clust\n' \
              '(Optimised consensus clustering of multiple heterogenous datasets)\n' \
-             'Python package version {0} (2018) Basel Abu-Jamous'.format(version[1:])
+             'Python package version {0} (2019) Basel Abu-Jamous'.format(version[1:])
     res += msgformated(tmptxt, alignment='^')
     res += midline()
 
@@ -94,8 +94,8 @@ def generateoutputsummaryparag(X, Xprocessed, Map, GDMall, GDM, uncle_res, mn_re
     endtime = dt.datetime.now()
     t = endtime - starttime
     d = t.days
-    h = t.seconds / 3600
-    m = (t.seconds % 3600) / 60
+    h = int(t.seconds / 3600)
+    m = int((t.seconds % 3600) / 60)
     s = t.seconds % 60
     if t.total_seconds() < 1:
         mil = t.microseconds / 1000
@@ -134,7 +134,7 @@ def generateoutputsummaryparag(X, Xprocessed, Map, GDMall, GDM, uncle_res, mn_re
                  'is {10} {1}s.' \
                  ''.format(label0, label1, L, dataset_datasets, Ng_original, Ng_to_clustering, K, Ng_in_clusters,
                            np.min(np.sum(B_corrected, axis=0)), np.max(np.sum(B_corrected, axis=0)),
-                           np.mean(np.sum(B_corrected, axis=0)))
+                           int(round(np.mean(np.sum(B_corrected, axis=0)))))
     else:
         tmptxt = 'Clust received {2} {3} with {4} unique {0}. After filtering, {5} {1}s made it to the clustering ' \
                  'step. Clust generated {6} clusters of {1}s.' \
@@ -148,17 +148,18 @@ def generateoutputsummaryparag(X, Xprocessed, Map, GDMall, GDM, uncle_res, mn_re
     res += midline()
 
     res += msgformated('Citation\n~~~~~~~~', alignment='^')
-    tmptxt = 'When publishing work that uses Clust, please include these two citations:\n' \
-             '1. Basel Abu-Jamous and Steven Kelly (2018) Clust: automatic extraction of optimal co-expressed ' \
-             'gene clusters from gene expression data. bioRxiv 221309; doi: https://doi.org/10.1101/221309.'
-
+    tmptxt = 'When publishing work that uses Clust, please include this citation:\n' \
+             'Basel Abu-Jamous and Steven Kelly (2018) Clust: automatic extraction of optimal co-expressed ' \
+             'gene clusters from gene expression data. Genome Biology 19:172; ' \
+             'doi: https://doi.org/10.1186/s13059-018-1536-8.'
+#TODO: citation
     res += msgformated(tmptxt)
     res += midline()
 
     tmptxt = 'For enquiries contact:\n' \
              'Basel Abu-Jamous\n' \
-             'Department of Plant Sciences, University of Oxford\n' \
-             'basel.abujamous@plants.ox.ac.uk\n' \
+             'Sensyne Health, Oxford, UK\n' \
+             'basel.abu-jamous@sensynehealth.com\n' \
              'baselabujamous@gmail.com'
     res += msgformated(tmptxt)
     res += bottomline(withnewline=False)
@@ -171,27 +172,50 @@ def summarise_results(X, Xprocessed, Map, GDMall, GDM, uncle_res, mn_res,
     K = B_corrected.shape[1]
     if K > 0:
         if Map is None:
-            res = collec.OrderedDict(
-                [('Starting data and time', starttime.strftime('%A %d %B %Y (%H:%M:%S)')),
-                 ('Ending date and time', endtime.strftime('%A %d %B %Y (%H:%M:%S)')),
-                 ('Time consumed', timeconsumedtxt),
-                 ('Number of datasets', len(X)),
-                 ('Total number of input genes', GDMall.shape[0]),
-                 ('Genes included in the analysis', GDM.shape[0]),
-                 ('Genes filtered out from the analysis', GDMall.shape[0] - GDM.shape[0]),
-                 ('Number of clusters', B_corrected.shape[1]),
-                 ('Total number of genes in clusters', np.sum(np.any(B_corrected, axis=1))),
-                 ('Genes not included in any cluster', B_corrected.shape[0] - np.sum(np.any(B_corrected, axis=1))),
-                 ('Min cluster size', np.min(np.sum(B_corrected, axis=0))),
-                 ('Max cluster size', np.max(np.sum(B_corrected, axis=0))),
-                 ('Average cluster size', np.mean(np.sum(B_corrected, axis=0))),
-                 ('nokey0', '**********************************'),
-                 ('Genes included in all datasets', np.sum(np.all(GDMall, axis=1))),
-                 ('Genes missed from 1 dataset', np.sum(np.sum(GDMall == 0, axis=1) == 1)),
-                 ('Genes missed from 2 datasets', np.sum(np.sum(GDMall == 0, axis=1) == 2)),
-                 ('Genes missed from 3 datasets', np.sum(np.sum(GDMall == 0, axis=1) == 3)),
-                 ('Genes missed from more than 3 datasets', np.sum(np.sum(GDMall == 0, axis=1) > 3))
-                 ])
+            if len(X) == 1:
+                res = collec.OrderedDict(
+                    [('Starting data and time', starttime.strftime('%A %d %B %Y (%H:%M:%S)')),
+                     ('Ending date and time', endtime.strftime('%A %d %B %Y (%H:%M:%S)')),
+                     ('Time consumed', timeconsumedtxt),
+                     ('Number of datasets', len(X)),
+                     ('Total number of input genes', GDMall.shape[0]),
+                     ('Genes included in the analysis', GDM.shape[0]),
+                     ('Genes filtered out from the analysis', GDMall.shape[0] - GDM.shape[0]),
+                     ('Number of clusters', B_corrected.shape[1]),
+                     ('Total number of genes in clusters', np.sum(np.any(B_corrected, axis=1))),
+                     ('Genes not included in any cluster', B_corrected.shape[0] - np.sum(np.any(B_corrected, axis=1))),
+                     ('Min cluster size', np.min(np.sum(B_corrected, axis=0))),
+                     ('Max cluster size', np.max(np.sum(B_corrected, axis=0))),
+                     ('Average cluster size', np.mean(np.sum(B_corrected, axis=0))),
+                     ('Number of samples (including replicates)', np.shape(X[0])[1]),
+                     ('Number of conditions', np.shape(Xprocessed[0])[1]),
+                     ])
+            else:
+                res = collec.OrderedDict(
+                    [('Starting data and time', starttime.strftime('%A %d %B %Y (%H:%M:%S)')),
+                     ('Ending date and time', endtime.strftime('%A %d %B %Y (%H:%M:%S)')),
+                     ('Time consumed', timeconsumedtxt),
+                     ('Number of datasets', len(X)),
+                     ('Total number of input genes', GDMall.shape[0]),
+                     ('Genes included in the analysis', GDM.shape[0]),
+                     ('Genes filtered out from the analysis', GDMall.shape[0] - GDM.shape[0]),
+                     ('Number of clusters', B_corrected.shape[1]),
+                     ('Total number of genes in clusters', np.sum(np.any(B_corrected, axis=1))),
+                     ('Genes not included in any cluster', B_corrected.shape[0] - np.sum(np.any(B_corrected, axis=1))),
+                     ('Min cluster size', np.min(np.sum(B_corrected, axis=0))),
+                     ('Max cluster size', np.max(np.sum(B_corrected, axis=0))),
+                     ('Average cluster size', np.mean(np.sum(B_corrected, axis=0))),
+                     ('Total number of samples (including replicates)', np.sum([np.shape(x)[1] for x in X])),
+                     ('Average number of samples (including replicates) per dataset', np.mean([np.shape(x)[1] for x in X])),
+                     ('Total number of conditions', np.sum([np.shape(x)[1] for x in Xprocessed])),
+                     ('Average number of conditions per dataset', np.mean([np.shape(x)[1] for x in Xprocessed])),
+                     ('nokey0', '**********************************'),
+                     ('Genes included in all datasets', np.sum(np.all(GDMall, axis=1))),
+                     ('Genes missed from 1 dataset', np.sum(np.sum(GDMall == 0, axis=1) == 1)),
+                     ('Genes missed from 2 datasets', np.sum(np.sum(GDMall == 0, axis=1) == 2)),
+                     ('Genes missed from 3 datasets', np.sum(np.sum(GDMall == 0, axis=1) == 3)),
+                     ('Genes missed from more than 3 datasets', np.sum(np.sum(GDMall == 0, axis=1) > 3))
+                     ])
         else:
             genesOGs = np.array([[len(og_species) for og_species in og] for og in Map]) # Number of genes per OG per species
             res = collec.OrderedDict(
@@ -208,6 +232,10 @@ def summarise_results(X, Xprocessed, Map, GDMall, GDM, uncle_res, mn_res,
                  ('Min cluster size', np.min(np.sum(B_corrected, axis=0))),
                  ('Max cluster size', np.max(np.sum(B_corrected, axis=0))),
                  ('Average cluster size', np.mean(np.sum(B_corrected, axis=0))),
+                 ('Total number of samples (including replicates)', np.sum([np.shape(x)[1] for x in X])),
+                 ('Average number of samples (including replicates) per dataset', np.mean([np.shape(x)[1] for x in X])),
+                 ('Total number of conditions', np.sum([np.shape(x)[1] for x in Xprocessed])),
+                 ('Average number of conditions per dataset', np.mean([np.shape(x)[1] for x in Xprocessed])),
                  ('nokey0', '**********************************'),
                  ('Number of species', Map.shape[1]),
                  ('Average number of genes in an OG per type', np.mean(genesOGs)),
@@ -241,6 +269,10 @@ def summarise_results(X, Xprocessed, Map, GDMall, GDM, uncle_res, mn_res,
                  ])
         else:
             genesOGs = np.array([[len(og_species) for og_species in og] for og in Map]) # Number of genes per OG per species
+            meanGenesOGs = 0 if len(genesOGs) == 0 else np.mean(genesOGs)
+            maxGenesOGs = 0 if len(genesOGs) == 0 else np.max(genesOGs)
+            OGsWithAtLeastASingleGeneInEachType = 0 if len(genesOGs) == 0 else np.sum(np.all(genesOGs >= 1, axis=1))
+            OGsWithExactlyASingleGeneInEachType = 0 if len(genesOGs) == 0 else np.sum(np.all(genesOGs == 1, axis=1))
             res = collec.OrderedDict(
                 [('Starting data and time', starttime.strftime('%A %d %B %Y (%H:%M:%S)')),
                  ('Ending date and time', endtime.strftime('%A %d %B %Y (%H:%M:%S)')),
@@ -252,10 +284,10 @@ def summarise_results(X, Xprocessed, Map, GDMall, GDM, uncle_res, mn_res,
                  ('Number of clusters', B_corrected.shape[1]),
                  ('nokey0', '**********************************'),
                  ('Number of species', Map.shape[1]),
-                 ('Average number of genes in an OG per type', np.mean(genesOGs)),
-                 ('Max number of genes in an OG per type', np.max(genesOGs)),
-                 ('Number of OGs with at least a single gene in each type', np.sum(np.all(genesOGs >= 1, axis=1))),
-                 ('Number of OGs with exactly a single gene in each type', np.sum(np.all(genesOGs == 1, axis=1))),
+                 ('Average number of genes in an OG per type', meanGenesOGs),
+                 ('Max number of genes in an OG per type', maxGenesOGs),
+                 ('Number of OGs with at least a single gene in each type', OGsWithAtLeastASingleGeneInEachType),
+                 ('Number of OGs with exactly a single gene in each type', OGsWithExactlyASingleGeneInEachType),
                  ('nokey1', '**********************************'),
                  ('OGs included in all datasets', np.sum(np.all(GDMall, axis=1))),
                  ('OGs missed from 1 dataset', np.sum(np.sum(GDMall == 0, axis=1) == 1)),
@@ -273,7 +305,7 @@ def clusters_genes_OGs(B, OGs, Map, MapSpecies, delim='; '):
         Nsp = len(MapSpecies)  # Number of species
     K = B.shape[1]  # Number of clusters
     if K == 0:
-        return np.array(np.empty([1, 1]), dtype=object)
+        return pd.DataFrame(data=np.array(np.empty([0, 0]), dtype=object), index=None, columns=None, dtype=str)
     Csizes = np.sum(B, axis=0)  # Clusters' sizes
     maxCsize = np.max(Csizes)  # Largest cluster size
     res = np.array(np.empty([maxCsize, (Nsp + 1) * K], dtype=str), dtype=object)
@@ -299,6 +331,13 @@ def clusters_genes_Species(B, OGs, Map, MapSpecies):
     Nsp = len(MapSpecies)  # Number of species
     K = B.shape[1]  # Number of clusters
 
+    # If no clusters, return a trivial output
+    if K == 0:
+        resFrames = np.array([None] * Nsp, dtype=object)
+        for sp in range(Nsp):
+            resFrames[sp] = pd.DataFrame(data=np.empty([0, 0]), columns=None, index=None, dtype=str)
+        return resFrames
+
     # Find flattened genes in species
     flatGenesInSpecies = [[ds.flattenAList(Map[B[:, k], sp]) for k in range(K)] for sp in range(Nsp)]  # Nsp x K lists
 
@@ -322,6 +361,46 @@ def clusters_genes_Species(B, OGs, Map, MapSpecies):
     return resFrames
 
 
+'''
+B: Ng x K
+OGs: Ng x 1
+Map: Ng x Nspecies
+MapSpecies: array of names of species in the Map (header of Map)
+'''
+def clusters_B_as_dataframes(B, OGs, Map):
+    clusterslabels = ['C{0}'.format(c) for c in range(np.shape(B)[1])]
+    B = pd.DataFrame(B, OGs, clusterslabels, dtype=bool)
+
+    if Map is None:
+        return B
+    else:
+        Nsp = np.shape(Map)[1]
+        PerSpecies_B = [None] * Nsp
+        for sp in range(Nsp):
+            RowNames = [ds.concatenateStrings(gs, ',') for gs in Map[:, sp]]  # Groups of genes
+            PerSpecies_B[sp] = pd.DataFrame(B, RowNames, clusterslabels, dtype=bool)
+
+        return B, PerSpecies_B
+
+
+'''
+B: Ng x K
+OGs: Ng x 1
+Map: Ng x Nspecies
+MapSpecies: array of names of species in the Map (header of Map)
+'''
+def processed_X_as_dataframes(X_processed, OGs, conditions):
+    L = len(X_processed)
+
+    Xout = [None] * L
+
+    for l in range(L):
+        Xout[l] = pd.DataFrame(X_processed[l], OGs, conditions[l])
+
+    return Xout
+
+
+
 def processed_X(Xprocessed, conditions, GDM, OGs, Map, MapSpecies):
     L = len(Xprocessed)
     res = np.array([None] * L, dtype=object)
