@@ -4,18 +4,17 @@ import sys
 
 import numpy as np
 
-import clustering as cl
-import datastructures as ds
-import io
-import mnplots as mn
-import numeric as nu
+import clust.scripts.clustering as cl
+import clust.scripts.datastructures as ds
+import clust.scripts.io as io
+import clust.scripts.mnplots as mn
+import clust.scripts.numeric as nu
 
 from joblib import Parallel, delayed
-import joblib
 import warnings
 import gc
-from glob import maxgenesinsetforpdist
-import output as op
+from clust.scripts.glob import maxgenesinsetforpdist
+import clust.scripts.output as op
 
 
 def binarise(U, technique, param=0.0):
@@ -309,10 +308,10 @@ def sortclusters(CoPaM, Mc, minGenesinClust = 11):
 
 
 # Clustering helping function for parallel loop
-def clustDataset(X, K, D, methods, GDMcolumn, Ng, datasetID):
-    #X, K, D, methods, GDMcolumn, Ng = args
+def clustDataset(X, K, methods, GDMcolumn, Ng, datasetID):
+    #X, K, methods, GDMcolumn, Ng = args
     Uloc = [np.zeros([Ng, K], dtype=bool)] * len(methods)  # Prepare the U output
-    tmpU = cl.clusterdataset(X, K, D, methods, datasetID)  # Obtain U's
+    tmpU = cl.clusterdataset(X, K, methods, datasetID)  # Obtain U's
     for cc in range(len(methods)):  # Set U's as per the GDM values
         Uloc[cc][GDMcolumn] = tmpU[cc]
     return Uloc
@@ -351,15 +350,9 @@ def uncles(X, type='A', Ks=[n for n in range(4, 21, 4)], params=None, methods=No
         methods = [['k-means']]
         # largest_DS = np.max([x.shape[0] for x in Xloc])
         # if (largest_DS <= maxgenesinsetforpdist):
-        #     if (deterministic):
-        #         methods = [['k-means'], ['HC']]
-        #     else:
-        #         methods = [['k-means'], ['SOMs'], ['HC']]
+        #    methods = [['k-means'], ['HC']]
         # else:
-        #     if (deterministic):
-        #         methods = [['k-means']]
-        #     else:
-        #         methods = [['k-means'], ['SOMs']]
+        #    methods = [['k-means']]
     else:
         largest_DS = np.max([x.shape[0] for x in Xloc])
         if (largest_DS > maxgenesinsetforpdist):
@@ -386,9 +379,45 @@ def uncles(X, type='A', Ks=[n for n in range(4, 21, 4)], params=None, methods=No
     if L > len(setsPloc):
         setsNloc = [ii for ii in range(len(setsPloc),L)]
 
-    Ds = [nu.closest_to_square_factors(k) for k in Ks]  # Grid sizes for the SOMs method for each value of K
+    Ks = np.array(Ks)
+    Ks = Ks[Ks <= Ng]  # Remove Ks that are larger than the number of genes Ng
+    Ks = Ks.tolist()
     NKs = len(Ks)  # Number of K values
 
+    # If the dataset is empty, return basic output
+    if Ng == 0:
+        NPp = len(binarise_paramP)  # Number of P params
+        NNp = len(binarise_paramN)  # Number of N params
+        if type == 'A':
+            B = np.zeros([CoPaMfinaltrials, NPp, 1, NKs], dtype=object)
+            Mc = np.zeros([CoPaMfinaltrials, NKs], dtype=object)
+        elif type == 'B':
+            B = np.zeros([CoPaMfinaltrials, NPp, NNp, NKs], dtype=object)
+            Mc = np.zeros([CoPaMfinaltrials, NKs], dtype=object)
+
+        params = dict(params, **{
+            'methods': methods,
+            'setsP': setsPloc,
+            'setsN': setsNloc,
+            'dofuzzystretch': dofuzzystretch,
+            'type': type,
+            'Ks': Ks,
+            'NKs': NKs,
+            'wsets': wsets,
+            'wmethods': wmethods,
+            'Ds': Ds,
+            'L': L,
+            'CoPaMs': np.array([None] * (CoPaMfinaltrials * NKs)).reshape([CoPaMfinaltrials, NKs]),
+            'smallestclustersize': smallestClusterSize,
+            'GDM': GDMloc
+        })
+
+        Uloc = np.array([None] * (L * NKs)).reshape([L, NKs])
+
+        UnclesRes = collections.namedtuple('UnclesRes', ['B', 'Mc', 'params', 'X', 'U'])
+        return UnclesRes(B, Mc, params, Xloc, Uloc)
+
+
     # Clustering
     if U is None:
         Utype = 'PM'
@@ -410,7 +439,7 @@ def uncles(X, type='A', Ks=[n for n in range(4, 21, 4)], params=None, methods=No
                 warnings.simplefilter("ignore")
                 Utmp = Parallel(n_jobs=ncores)\
                     (delayed(clustDataset)
-                     (Xloc[l], Ks[ki], Ds[ki], methodsDetailedloc[l], GDMloc[:, l], Ng, l) for ki in range(NKs))
+                     (Xloc[l], Ks[ki], methodsDetailedloc[l], GDMloc[:, l], Ng, l) for ki in range(NKs))
 
                 Utmp = [u for u in Utmp]
                 for ki in range(NKs):
@@ -544,7 +573,6 @@ def uncles(X, type='A', Ks=[n for n in range(4, 21, 4)], params=None, methods=No
         'NKs': NKs,
         'wsets': wsets,
         'wmethods': wmethods,
-        'Ds': Ds,
         'L': L,
         'CoPaMs': CoPaMs,
         'smallestclustersize': smallestClusterSize,
